#!/bin/bash
#set -x
testname=`basename $0`
SPATH="/sys/devices/system/cpu"
TPATH="./wlink"
HWCHECK_LIST=
ERR_FLAG=0
INITD_SLIM=/etc/init.d/slim
DIFF_MODE=0
TIME=10

usage()
{
cat >&2 <<EOL
Usage:  $testname [-d|-h] [-t <sec>]
	default mode (error number is measured)
	d - error difference is measured
	t - time (sec), default 30 sec
 	h - help
EOL
exit 0
}

check_hw_check_driver()
{

	local i=0
	local HW_PATH

	lsmod | grep "hw_check" > /dev/null
	if (($?)); then
		modprobe hw_check 2>/dev/null
		if (($?)); then
			echo "ERROR: hw_check cannot be loaded .."
			exit 1
		fi
	fi

	HWCHECK_LIST=$(ls $SPATH | grep hw_check)
	if [[ -z $HWCHECK_LIST ]]; then
		echo "ERROR: Not found hw_check device .."
		exit 1
	fi

	for HWCHECK_DEV in $HWCHECK_LIST; do
		DRIVER_PATHS[$i]="$SPATH/$HWCHECK_DEV"
		HW_PATH=$(realpath $SPATH/$HWCHECK_DEV/hwmon/hwmon*/)
		if [[ ! -f "$HW_PATH/link_info" ]]; then
			echo "ERROR: No link_info file .."
			exit 1
		fi
	done
	echo "--------------------------------------------------------------"
}

get_def_wlcc_data()
{
	local i=0
	local STATUS_INFO
	local LINK_INFO
	local STR

	echo -e "READ (default wlcc data)\n"
    	for HWCHECK_DEV in $HWCHECK_LIST; do
        	LINK_INFO=$(realpath $SPATH/$HWCHECK_DEV/hwmon/hwmon*/link_info)
        	STR=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $9}')
        	ACTIVE[$i]=${STR:7:1}

	        STR=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $10}')
        	STATE[$i]=${STR:6:1}

       		if [[ ${ACTIVE[$i]} != "1" ]] || [[ ${STATE[$i]} != "3" ]]; then
			if [[ ${ACTIVE[$i]} != "1" ]] && [[ ${STATE[$i]} != "3" ]]; then
				STATUS_INFO="failed! state is not work; node is not active"
        		elif [[ ${ACTIVE[$i]} != "1" ]]; then
            			STATUS_INFO="failed! node is not active"
			else
            			STATUS_INFO="failed! state is not work"
			fi
           		print_str "NODE:" $i "<---- $STATUS_INFO\n"
        		i=$(($i+1))
            		continue
        	fi

        	WIDTH[$i]=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $12}'| awk -F= {'print $2'})
        	ERR_CNT[$i]=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $17}'| awk -F= {'print $2'})
		print_str "NODE:" $i
		print_str "ACTIVE:" ${ACTIVE[$i]}
		print_str "STATE: " ${STATE[$i]}
		print_str "WIDTH:" ${WIDTH[$i]}


        	if [[ ${ERR_CNT[$i]} == 0x0 ]]; then
        		print_str "ERRORS:" ${ERR_CNT[$i]}
        	else
			if [[ $DIFF_MODE == 1 ]]; then
				print_str "ERRORS:" ${ERR_CNT[$i]} "<---- WARNING"
			else
				ERR_FLAG=1
				print_str "ERRORS:" ${ERR_CNT[$i]} "<---- ERROR"
			fi
		fi
		echo
        	i=$(($i+1))
	done
	echo "--------------------------------------------------------------"
}

wait_xfmw4()
{
	for ((i=0; i<20; i++)); do
		if (pidof xfwm4) >/dev/null; then
			break
		fi
		sleep 0.5
	done
}

wait_x () (
    for ((i=0; i<20; i++)); do
	(set -x; pidof X) && break
        sleep .5
    done
)

slim_setting()
{
	if [[ ! -f $INITD_SLIM ]]; then
		echo -e "\t!!! Х-SERVER IS NOT RUNNING !!!"
		exit 1
	fi
	#Autologin enable
	SLIM_CONF=/etc/slim.conf
	SLIM_CONFB=${SLIM_CONF}-BASE
	SLIM_CONFA=${SLIM_CONF}-AUTO_LOGIN
	if [[ ! -f $SLIM_CONFB ]]; then
		(set -x; cp -p $SLIM_CONF $SLIM_CONFB)
	fi
	diff -q $SLIM_CONFA $SLIM_CONF >&/dev/null
	if (($?)); then
		(set -x
		cp $SLIM_CONFB $SLIM_CONFA
		echo -e "\ndefault_user\troot\nfocus_password\tyes\nauto_login\tyes"\
			                                            >> $SLIM_CONFA
		cp $SLIM_CONFA $SLIM_CONF
		)
	fi
	SLIM_AUTOLOGIN=1
}

run_test()
{
	export DISPLAY=:0

	echo -e "TEST\n"
	#Check if X is running
	if (! pidof X && ! pidof Xorg) >/dev/null; then
		slim_setting

		$INITD_SLIM start
		wait_x

		if (! pidof X && ! pidof Xorg) >/dev/null; then
			echo -e "\t!!! Х-SERVER IS NOT RUNNING !!!"
			cp $SLIM_CONFB $SLIM_CONF
			exit 1
		fi
		#Wait for graphics
		wait_xfmw4
		if (! pidof xfmw4) >/dev/null; then
			echo -e "\t!!! ХFMW IS NOT RUNNING !!!"
			cp $SLIM_CONFB $SLIM_CONF
			exit 1
		fi
	else
		xset s 0 >/dev/null 2>&1
		xset -dpms >/dev/null 2>&1
		#Check log in
		if (! pidof xfwm4) >/dev/null; then
			slim_setting
			$INITD_SLIM restart
			wait_xfmw4
		fi
	fi

	$TPATH -t $TIME

	#Return config to its initial state
	if [[ ! -z $SLIM_AUTOLOGIN ]]; then
		cp $SLIM_CONFB $SLIM_CONF
	fi
	echo "--------------------------------------------------------------"
}

get_wlcc_data()
{
	local i=0
	local LINK_INFO
	local STR

	for HWCHECK_DEV in $HWCHECK_LIST; do
        	LINK_INFO=$(realpath $SPATH/$HWCHECK_DEV/hwmon/hwmon*/link_info)
        	STR=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $9}')
        	CUR_ACTIVE[$i]=${STR:7:1}
        	STR=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $10}')
        	CUR_STATE[$i]=${STR:6:1}
        	CUR_WIDTH[$i]=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $12}'| awk -F= {'print $2'})
		CUR_ERR_CNT[$i]=$(cat "$LINK_INFO" | grep "wlcc" | awk '{print $17}'| awk -F= {'print $2'})

        	i=$(($i+1))
    	done
}

print_str()
{
	local TEMP_STR=$(printf "%-10s %-10s %s " "$1" "$2" "$3")
	echo -e "$TEMP_STR"
}

compare_wlcc_data()
{
	local i=0
	local DIFF_ERR_CNT
	local STATUS_INFO

	echo -e "READ (wlcc data); COMPARE\n"
    	for HWCHECK_DEV in $HWCHECK_LIST; do
		if [[ ${CUR_ACTIVE[$i]} != "1" ]] || [[ ${CUR_STATE[$i]} != "3" ]]; then
			if [[ ${CUR_ACTIVE[$i]} != "1" ]] && [[ ${CUR_STATE[$i]} != "3" ]]; then
				STATUS_INFO="STATE IS NOT WORK; NODE IS NOT ACTIVE"
			elif [[ ${CUR_ACTIVE[$i]} != "1" ]]; then
				STATUS_INFO="NODE IS NOT ACTIVE"
			else
				STATUS_INFO="STATE IS NOT WORK"
			fi
			print_str "NODE:" $i "<---- $STATUS_INFO\n"
			i=$(($i+1))
			continue
		fi

		print_str "NODE:" $i
		print_str "ACTIVE:" ${CUR_ACTIVE[$i]}

		if [[ ${CUR_STATE[$i]} != ${STATE[$i]} ]]; then
			print_str "STATE:" ${CUR_STATE[$i]} "<---- DIFF"
		else
			print_str "STATE:" ${CUR_STATE[$i]}
		fi

		if [[ ${CUR_WIDTH[$i]} != ${WIDTH[$i]} ]]; then
			print_str "WIDTH:" ${CUR_WIDTH[$i]} "<---- DIFF"
		else
			print_str "WIDTH:" ${CUR_WIDTH[$i]}
		fi
		if [[ ${CUR_ERR_CNT[$i]} == 0x0 ]]; then
			print_str "ERRORS:" ${CUR_ERR_CNT[$i]}
		else
			if [[ $DIFF_MODE == 1 ]] && [[ $((CUR_ERR_CNT[$i])) > $((ERR_CNT[$i])) ]]; then
				print_str "ERRORS:" ${CUR_ERR_CNT[$i]} "<---- ERROR"
			elif [[ $DIFF_MODE == 1 ]]; then
				print_str "ERRORS:" ${CUR_ERR_CNT[$i]} "<---- WARNING"
			else
				ERR_FLAG=1
				print_str "ERRORS:" ${CUR_ERR_CNT[$i]} "<---- ERROR"
			fi
		fi

		if [[ $DIFF_MODE == 1 ]]; then
			if [[ $((ERR_CNT[$i])) > $((CUR_ERR_CNT[$i])) ]]; then
				print_str "ERR_DIFF:" "failed! initial number of errors is greater than current .."
				ERR_FLAG=1
			else
				DIFF_ERR_CNT=`printf "0x%x" $(($((CUR_ERR_CNT[$i])) - $((ERR_CNT[$i]))))`
				print_str "ERR_DIFF:" $DIFF_ERR_CNT
				if (($DIFF_ERR_CNT)); then
					ERR_FLAG=1
				fi
			fi
		fi
		echo
		i=$(($i+1))
    done
}

OPTIONS="hdt:"
while getopts $OPTIONS OPT
do
	case $OPT in
		h ) usage; exit 0;;
		d ) DIFF_MODE=1;;
		t ) TIME=$OPTARG;;
		* ) echo "incorrect param: $OPT"; usage;;
	esac
done

if [[ $DIFF_MODE == 1 ]]; then
	echo "Running: DIFF mode"
else
	echo "Running: DEFAULT mode"
fi

check_hw_check_driver
get_def_wlcc_data
run_test
get_wlcc_data
compare_wlcc_data

if [[ $ERR_FLAG == 1 ]]; then
	exit 1
fi

exit 0
